@@ -1,11 +1,11 @@
 
-# $Id: Ebay.pm,v 2.270 2015-06-06 20:45:52 Martin Exp $
-
 package WWW::Search::Ebay;
 
 use strict;
 use warnings;
 
+our $VERSION = 2.274;
+
 =head1 NAME
 
 WWW::Search::Ebay - backend for searching www.ebay.com
@@ -135,8 +135,6 @@ use WWW::Search qw( generic_option strip_tags );
 use WWW::SearchResult 2.072;
 use WWW::Search::Result;
 
-our
-$VERSION = do { my @r = (q$Revision: 2.270 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 our $MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
 my $cgi = new CGI;
 
@@ -526,7 +524,14 @@ sub _parse_shipping
   # I don't know why there are sometimes weird characters in there:
   $iPrice =~ s!&Acirc;!!g;
   $iPrice =~ s!!!g;
+  $iPrice =~ s!\+!!g;
+  $iPrice =~ s/SHIPPING//gi;
   print STDERR " DDD   raw shipping ===$iPrice===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+  if ($iPrice =~ m/UNKNOWN/i)
+    {
+    # No shipping info provided:
+    return 1;
+    } # if
   if ($iPrice =~ m/FREE/i)
     {
     $iPrice = 0.00;
@@ -692,10 +697,15 @@ sub _get_result_count_elements
   {
   my $self = shift;
   my $tree = shift;
-  my @ao = $tree->look_down(
-                            '_tag' => 'div',
-                            class => 'fpcc'
-                           );
+  my @ao;
+  push @ao, $tree->look_down( # as of 2015-06
+                             '_tag' => 'span',
+                             class => 'listingscnt'
+                            );
+  push @ao, $tree->look_down(
+                             '_tag' => 'div',
+                             class => 'fpcc'
+                            );
   push @ao, $tree->look_down(
                              '_tag' => 'div',
                              class => 'fpc'
@@ -717,9 +727,9 @@ sub _get_result_count_elements
                             '_tag' => 'div',
                             id => 'rsc'
                            );
-  push @ao, $tree->look_down( # as of 2015-06
-                            '_tag' => 'span',
-                            class => 'listingscnt'
+  push @ao, $tree->look_down( # for Category, as of 2018-02:
+                            _tag => 'h2',
+                            class => 'srp-controls__count-heading'
                            );
   return @ao;
   } # _get_result_count_elements
@@ -763,6 +773,16 @@ sub _get_itemtitle_tds
                                class => 'lvtitle',
                               );
     } # if
+  # This is for Category search as of 2018-02:
+  $oDiv = $tree->look_down(_tag => 'ul',
+                           class => 'b-list__items_nofooter',
+                          );
+  if (ref $oDiv)
+    {
+    push @ao, $oDiv->look_down(_tag => 'div',
+                               class => 's-item__info clearfix',
+                              );
+    } # if
   return @ao;
   } # _get_itemtitle_tds
 
@@ -845,24 +865,35 @@ sub _parse_tree
       } # if
     } # if
 
-  # The hit count is in a FONT tag:
-  my @aoFONT = $self->_get_result_count_elements($tree);
- FONT:
-  foreach my $oFONT (@aoFONT)
+  my $iHits = 0;
+
+  # The hit count is in one of these tags:
+  my @aoResultCountTagset = $self->_get_result_count_elements($tree);
+  if (scalar(@aoResultCountTagset) < 1)
     {
+    warn " EEE no result_count_elements matched the HTML\n";
+    } # if
+ FONT:
+  foreach my $oFONT (@aoResultCountTagset)
+    {
+    my $s = $oFONT->as_text;
     my $qr = $self->_result_count_pattern;
-    print STDERR (" DDD   result_count try ==",
-                  $oFONT->as_text, "== against qr=$qr=\n") if (1 < $self->{_debug});
-    if ($oFONT->as_text =~ m!$qr!)
+    print STDERR (" DDD   result_count try qr=$qr= against ==$s==\n") if (1 < $self->{_debug});
+    if ($s =~ m/$qr/)
       {
       my $sCount = $1;
       print STDERR " DDD     matched ($sCount)\n" if (1 < $self->{_debug});
       # Make sure it's an integer:
       $sCount =~ s!,!!g;
-      $self->approximate_result_count($sCount);
+      $self->approximate_result_count(0 + $sCount);
       last FONT;
       } # if
-    } # foreach
+    } # foreach FONT
+
+  if ($self->approximate_result_count() < 1)
+    {
+    return $iHits;
+    } # if
 
   # Recursively parse the stats telling how many items were found in
   # each category:
@@ -890,7 +921,6 @@ sub _parse_tree
     # use File::Slurp;
     # write_file('no-results.html', $self->{response}->content);
     } # unless
-  my $iHits = 0;
   my $qrItemNum = qr{(\d{11,13})};
  TD:
   foreach my $oTDtitle (@aoTD)
@@ -961,7 +991,7 @@ sub _parse_tree
         {
         next TD if ! $self->_parse_enddate($oTDsib, $hit);
         }
-      if ($sColumn =~ 'time')
+      if ($sColumn =~ m'time')
         {
         next TD if ! $self->_parse_enddate($oTDsib, $hit);
         }
@@ -1003,6 +1033,15 @@ sub _parse_tree
     push(@{$self->{cache}}, $hit);
     $self->{'_num_hits'}++;
     $iHits++;
+
+    if ($self->approximate_result_count() <= $iHits)
+      {
+      # We've already scraped all the hits that eBay said there were.
+      # Stop scraping, so that we don't return hits that are not an
+      # exact match to the query but are just "related":
+      last TD;
+      } # if
+
     # Delete this HTML element so that future searches go faster?
     $oTDtitle->detach;
     $oTDtitle->delete;
@@ -1184,7 +1223,7 @@ Include parentheses so that $1 becomes the number (with commas is OK).
 
 sub _result_count_pattern
   {
-  return qr'([0-9,]+)\s+(active\s+)?(listing|item|matche?|result)s?(\s+found)?';
+  return qr'([0-9,]+)\s+(active\s+)?(listing|item|matche?|result)s?(\s+found)?'i;
   } # _result_count_pattern
 
 
